@@ -27,6 +27,10 @@
 #ifndef DOODLE_H
 #define DOODLE_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <stdio.h>
 
 /**
@@ -191,4 +195,10 @@ int DOODLE_tree_search_approx(struct DOODLE_SuffixTree * tree,
 void DOODLE_tree_set_memory_limit(struct DOODLE_SuffixTree * tree,
 				  size_t limit);
 
+
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif
