@@ -1,6 +1,6 @@
 /*
      This file is part of doodle.
-     (C) 2004 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2006 Christian Grothoff (and other contributing authors)
 
      doodle is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -27,7 +27,7 @@
  * tree.c.  doodle uses GNU getopt and for portability ships with its
  * own version of the getopt code (getopt*).<p>
  *
- * doodle requires libextractor (http://ovmj.org/libextractor/).
+ * doodle requires libextractor (http://gnunet.org/libextractor/).
  */
 
 #include "config.h"
@@ -39,6 +39,7 @@
 #include <netinet/in.h>
 #include <stdarg.h>
 #include <dirent.h>
+#include <signal.h>
 #include <fam.h>
 #include <locale.h>
 #include "doodle.h"
@@ -120,10 +121,11 @@ static int isPruned(const char * filename) {
 /**
  * Print log-messages to the logfile.
  */
-static void my_log(FILE * logfile,
+static void my_log(void * ctx,
 		   unsigned int level,
 		   const char * msg,
 		   ...) {
+  FILE * logfile = ctx;
   va_list args;
   if (logfile == NULL) {
     if (do_debug)
@@ -147,7 +149,7 @@ static void my_log(FILE * logfile,
  * @brief closure for do_index and other processing functions
  */
 typedef struct {
-  EXTRACTOR_ExtractorList * elist;
+  struct EXTRACT_Process * elist;
   struct DOODLE_SuffixTree * tree;
   FAMConnection fc;
   FAMRequest * fr;
@@ -175,7 +177,8 @@ typedef struct {
  * to process FAM events.  Puts all events into the
  * queue for the main worker.
  */
-static void * processEvents(DIC * cls) {
+static void * processEvents(void * arg) {
+  DIC * cls = arg;
   FAMEvent fe;
   fd_set r;
   fd_set w;
@@ -323,8 +326,8 @@ static int do_index(const char * filename,
     }
   }
 
-  if (0 != stat(filename,
-		&sbuf)) {
+  if (0 != lstat(filename,
+		 &sbuf)) {
     dic->log(dic->logContext,
 	     DOODLE_LOG_VERY_VERBOSE,
 	     _("Call to '%s' for file '%s' failed: %s\n"),
@@ -381,11 +384,6 @@ static int do_index(const char * filename,
 	     _("Will monitor directory '%s' for changes.\n"),
 	     filename);
     fn = STRDUP(filename);
-
-    /* make sure FAM queue is empty, otherwise the
-       next call may block forever; we might want
-       to do this with threading, but for that
-       we would need to start pthreads... */
     if (0 == FAMMonitorDirectory(&dic->fc,
 				 filename,
 				 &dic->fr[dic->frPos],
@@ -433,6 +431,7 @@ static int do_index(const char * filename,
 	     _("Processing file '%s'.\n"),
 	     filename);
     return buildIndex(dic->elist,
+		      NULL, 
 		      filename,
 		      dic->tree,
 		      do_filenames);
@@ -445,7 +444,8 @@ static int do_index(const char * filename,
 /**
  * Main worker thread.  Register FAM events and process.
  */
-static void * worker(DIC * cls) {
+static void * worker(void * arg) {
+  DIC * cls = arg;
   int i;
   int more;
   int wasMore;
@@ -462,7 +462,7 @@ static void * worker(DIC * cls) {
   cls->events = NULL;
   cls->signal = SEMAPHORE_NEW(0);
   if (0 != PTHREAD_CREATE(&helperThread,
-			  (PThreadMain) &processEvents,
+			  &processEvents,
 			  cls,
 			  64 * 1024)) {
     cls->log(cls->logContext,
@@ -472,9 +472,6 @@ static void * worker(DIC * cls) {
     return NULL;
   }
 
-  DOODLE_tree_truncate_modified(cls->tree,
-				cls->log,
-				cls->logContext);
   cls->log(cls->logContext,
 	   DOODLE_LOG_VERBOSE,
 	   _("Registering with FAM for file system events.\n"));
@@ -547,6 +544,12 @@ static void * worker(DIC * cls) {
   } /* forever (until signal) */
 
   cls->continueRunning = 0;
+  if (0 != FAMClose(&cls->fc)) {
+    cls->log(cls->logContext,
+	   DOODLE_LOG_CRITICAL,
+	   _("Error disconnecting from fam.\n"));
+  }
+  PTHREAD_KILL(&helperThread, SIGTERM);
   PTHREAD_JOIN(&helperThread, &unused);
   SEMAPHORE_FREE(cls->signal);
 
@@ -567,13 +570,12 @@ static int build(const char * libraries,
 		 int argc,		
 		 char * argv[]) {
   int i;
-  int ret;
+  unsigned int ret;
   DIC cls;
   char * ename;
   FILE * logfile;
   PTHREAD_T workerThread;
   void * unused;
-  FAMEvent fe;
 
   cls.argc = argc;
   cls.argv = argv;
@@ -590,7 +592,7 @@ static int build(const char * libraries,
 	     strerror(errno));
   }
   cls.logContext = logfile;
-  cls.log = (DOODLE_Logger) &my_log;
+  cls.log = &my_log;
 
 
   if (dbName == NULL) {
@@ -613,7 +615,7 @@ static int build(const char * libraries,
   if (ename == NULL)
     return -1;
   cls.ename = ename;
-  cls.tree = DOODLE_tree_create((DOODLE_Logger) &my_log,
+  cls.tree = DOODLE_tree_create(&my_log,
 				logfile,
 				ename);
   cls.treePresent = 1;
@@ -622,25 +624,11 @@ static int build(const char * libraries,
   if (mem_limit != 0)
     DOODLE_tree_set_memory_limit(cls.tree,
 				 mem_limit);
-  if (do_default) {
-    my_log(logfile,
-	   DOODLE_LOG_VERY_VERBOSE,
-	   _("Loading default set of libextractor plugins.\n"));
-    cls.elist = EXTRACTOR_loadDefaultLibraries();
-  } else
-    cls.elist = NULL;
-  if (libraries != NULL) {
-    my_log(logfile,
-	   DOODLE_LOG_VERY_VERBOSE,
-	   _("Loading libextractor plugins: '%s'\n"),
-	   libraries);
-    cls.elist = EXTRACTOR_loadConfigLibraries(cls.elist,
-					      libraries);
-  }
+  cls.elist = forkExtractor(do_default,
+			    libraries,
+			    &my_log,
+			    logfile);
   if (cls.elist == NULL) {
-    my_log(logfile,
-	   DOODLE_LOG_CRITICAL,
-	   _("Failed to load any libextractor plugins. Aborting.\n"));
     DOODLE_tree_destroy(cls.tree);
     return -1;
   }
@@ -667,7 +655,7 @@ static int build(const char * libraries,
 
   MUTEX_CREATE(&cls.lock);
   if (0 != PTHREAD_CREATE(&workerThread,
-			  (PThreadMain)&worker,
+			  &worker,
 			  &cls,
 			  64 * 1024)) {
     my_log(logfile,
@@ -677,6 +665,8 @@ static int build(const char * libraries,
     ret = -1;
   } else {
     wait_for_shutdown();
+    cls.continueRunning = 0;
+    SEMAPHORE_UP(cls.signal);
     PTHREAD_JOIN(&workerThread, &unused);
   }
   MUTEX_DESTROY(&cls.lock);
@@ -692,30 +682,11 @@ static int build(const char * libraries,
 
 
   for (i=0;i<cls.frSize;i++) {
-    /* flush fam-event queue */
-    while (FAMPending(&cls.fc))
-      if (-1 == FAMNextEvent(&cls.fc, &fe)) {
-	cls.log(cls.logContext,
-		DOODLE_LOG_CRITICAL,
-		_("Call to '%s' failed: %s\n"),
-		"FAMNextEvent",
-		FamErrlist[FAMErrno]);
-	break; /* avoid persistent error */
-      }
     if (cls.frNames[i] != NULL) {
       my_log(logfile,
 	     DOODLE_LOG_VERBOSE,
 	     _("Cancelling fam monitor '%s'.\n"),
 	     cls.frNames[i]);
-      if (-1 == FAMCancelMonitor(&cls.fc,
-				 &cls.fr[i])) {
-	cls.log(cls.logContext,
-		DOODLE_LOG_CRITICAL,
-		_("Call to '%s' for file '%s' failed: %s\n"),
-		"FAMCancelMonitor",
-		cls.frNames[i],
-		FamErrlist[FAMErrno]);
-      }
       free(cls.frNames[i]);
     }
   }
@@ -729,18 +700,14 @@ static int build(const char * libraries,
   GROW(cls.fr,
        cls.frSize,
        0);
+  cls.frSize = i;
   GROW(cls.frNames,
-       i,
+       cls.frSize,
        0);
-  if (0 != FAMClose(&cls.fc)) {
-    my_log(logfile,
-	   DOODLE_LOG_CRITICAL,
-	   _("Error disconnecting from fam.\n"));
-  }
   my_log(logfile,
 	 DOODLE_LOG_VERBOSE,
 	 _("Unloading libextractor plugins.\n"));
-  EXTRACTOR_removeAll(cls.elist);
+  joinExtractor(cls.elist);
   free(ename);
   if (logfile != NULL)
     fclose(logfile);
